"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const typescript_1 = __importDefault(require("typescript"));
const path_utils_1 = require("./path.utils");
const clean_addon_1 = __importDefault(require("./clean-addon"));
/**
 * Compile ts files by creating a compilation object using the compiler API and emitting js files.
 * @public
 */
function build(options) {
    const program = createProgramFromConfig(options);
    emit(program, options);
}
exports.build = build;
/**
 * Create a compilation object using the compiler API.
 * @public
 */
function createProgramFromConfig({ basePath, configFilePath, compilerOptions, include, exclude, files, extends: extend, references, }) {
    let config = {};
    if (configFilePath) {
        configFilePath = path_utils_1.ensureAbsolutePath(configFilePath, basePath);
        console.log(`Retrieving ${configFilePath}`);
        const readResult = typescript_1.default.readConfigFile(configFilePath, typescript_1.default.sys.readFile);
        if (readResult.error)
            logDiagnostics([readResult.error], true);
        config = readResult.config;
    }
    config.compilerOptions = Object.assign({}, config.compilerOptions, compilerOptions);
    if (include)
        config.include = include;
    if (exclude)
        config.exclude = exclude;
    if (files)
        config.files = files;
    if (extend)
        config.extends = extend;
    if (references)
        config.references = references;
    const { options, fileNames, projectReferences, errors } = typescript_1.default.parseJsonConfigFileContent(config, typescript_1.default.sys, basePath, undefined, configFilePath);
    logDiagnostics(errors, true);
    const program = typescript_1.default.createProgram({
        options,
        rootNames: fileNames,
        projectReferences,
    });
    return program;
}
exports.createProgramFromConfig = createProgramFromConfig;
/**
 * Compile Typescript files and emit diagnostics if any, throws an error if it fails.
 * @public
 */
function emit(program, { betterDiagnostics, clean, basePath } = {}) {
    const options = program.getCompilerOptions();
    if (clean) {
        clean_addon_1.default(clean, options, basePath);
    }
    if (options.listFiles) {
        console.log('Files to compile:\n' + program.getRootFileNames().join('\n'));
    }
    console.log('Compilation started');
    const { diagnostics, emitSkipped, emittedFiles } = program.emit();
    if (options.listEmittedFiles && emittedFiles) {
        console.log('Emitted files:\n' + emittedFiles.join('\n'));
    }
    // https://github.com/dsherret/ts-morph/issues/384
    const allDiagnostics = typescript_1.default.getPreEmitDiagnostics(program).concat(diagnostics);
    logDiagnostics(allDiagnostics, betterDiagnostics);
    if (!options.noEmit && emitSkipped)
        throw Error('Compilation failed');
    console.log('Compilation successful');
}
exports.emit = emit;
/**
 * Log compiler diagnostics to stderr.
 * @internal
 */
function logDiagnostics(diagnostics, better = false) {
    if (!diagnostics.length)
        return;
    const formatHost = {
        getCanonicalFileName: (path) => path,
        getCurrentDirectory: typescript_1.default.sys.getCurrentDirectory,
        getNewLine: () => typescript_1.default.sys.newLine,
    };
    const message = better
        ? typescript_1.default.formatDiagnosticsWithColorAndContext(diagnostics, formatHost)
        : typescript_1.default.formatDiagnostics(diagnostics, formatHost);
    console.warn(message);
}
